#' @import tibble
#' @import dplyr
NULL

#' Build error matrix.
#' 
#' Error matrix is a 4 x 4 numeric matrix providing sequencing
#' error probalibiltie for each nucleotides. Each row correspond
#' to one of the four nucleotides `A`,`C`,`G`,`T`. Each column indicates
#' the probability of reading a given nucleotide considering that the TRUE
#' one is indicated by the row.
#' `error.jc` build an error matrix following the Juke et Cantor model
#' where all errors are equiprobable.
#
#' @param rate is the global error probability per site expressed as a numeric
#'             value ranging between 0 and 1.
#'             
#' @return a 4 x 4 numeric matrix providing sequencing
#'         error probalibiltie for each nucleotides.
#'         
#' @examples 
#'   # Build an error matrix corresponding 
#'   # to 5% of sequencing error
#'   
#'   err = error_jc(0.05)
#'   err
#'   
#'   #          A          C          G          T
#'   # A 0.95000000 0.01666667 0.01666667 0.01666667
#'   # C 0.01666667 0.95000000 0.01666667 0.01666667
#'   # G 0.01666667 0.01666667 0.95000000 0.01666667
#'   # T 0.01666667 0.01666667 0.01666667 0.95000000
#'   
#' @author Eric Coissac
#' @export
error_jc = function(rate) {
  e = rate/3
  err = matrix(e,nrow = 4,ncol=4)
  diag(err)=1-rate
  rownames(err) <- c("A","C","G","T")
  colnames(err) <- rownames(err)
  
  return(err)
}

#' Generate substitution sequencing error.
#' 
#' 
#' @export
sequencing_error = function(nuc,error) {
  apply(error[nuc,],
        MARGIN = 1,
        function(p) sample(colnames(error),
                           size = 1,
                           prob = p))
}


#' @export
simulate_sites = function(haplotype.1,haplotype.2,coverage,error) {
  n = length(haplotype.1)
  stopifnot(n==length(haplotype.2))
  
  genotypes = tibble(haplotype.1,haplotype.2)
  genotypes = as.tibble(t(apply(genotypes,
                                MARGIN = 1,
                                sort)))
  
  cov = rpois(n,lambda = coverage)
  obs1= vapply(cov,function(x) rbinom(1,x,0.5),0)
  obs2 = cov - obs1
  
  rep = matrix(0,nrow = n,ncol = 4)
  colnames(rep) = c("A","C","G","T")

  mapply(function(i,a1,a2,o1,o2) { rep[i,a1]<<-rep[i,a1] + o1;
                                   rep[i,a2]<<-rep[i,a2] + o2},
               1:n,
               genotypes$haplotype.1,
               genotypes$haplotype.2,
               obs1,
               obs2)
  
  rep = apply(rep,
              MARGIN = 1,
              function(ns) table(factor(sequencing_error(rep(c("A","C","G","T"),
                                                             ns),
                                                         error),
                                        levels=c("A","C","G","T"))))
  
  rep = as.tibble(t(rep))
  
  rep = bind_cols(haplotype.1=haplotype.1,
                  haplotype.2=haplotype.2,
                  coverage=cov,
                  rep)
  
  return(rep)
}
